/*
  testterm.c

  June 18, 2015
  Code placed in Public Domain.  
  A device/sensor handler for the Raspberry Pi.
  Start out with a Sensirion sht11 module from Parallax.
  Moved to a DNA-RTC board for better voltage handling.
  Prototype for eventual(maybe) switch to a Propeller HAT.
  Things that may be added:
    -XBee module (added 6/22/15)
    -RTC (added 6/22/15)
    -DS18B20 temperature sensor, multiple modules.
    -CO2 sensor
    -How much can the Propeller HAT support?
*/
#include "simpletools.h"
#include "simpletext.h"
#include "fdserial.h"
#include "sht11.h"
#include "basic_i2c_driver.h"
#include "sRTC.h"

#define BUFFERLEN 10

serial *rpi;
serial *xbee;

/* Sensirion sht11 pins */
int  DatPin = 1;
int  ClkPin = 3;

sht11values_t MyVals;
sht11values_t *pMyVals;

volatile long Hum;
volatile long TmpC;
volatile long TmpF;
volatile long Status;
/***********************/

void Sensirion();  // New COG
void talkXBee();

int prompttime(char *prompt);
void Set_Clock();
void GetTime();
void GetTD();
void Set_Time();
void xbeeHelp();

int main()
{
  // Add startup code here.
  cog_run(talkXBee,256);
  pause(50);
  cog_run(Sensirion,128);  // Start the sht11 COG.
  pause(200);
/*                     Rx Tx         */
  rpi = fdserial_open(19,20,0,115200);  // Raspberry Pi in this case.
  pause(50);

  char inBuff[40];

  //int message = 9999;  // Debug
  char outBuff[10];
     
  while(1)
  {
    // Add main loop code here.
      readStr(rpi, inBuff, 40);         // Waiting for incoming
      if(!strcmp(inBuff,"gettempf"))        // Checks the string
      {
        //sprintf(outBuff,"%d",message);  // Converts to ascii, Debug
        sprintf(outBuff,"%d",TmpF/10-3); // Use appropriate "%" for data
        writeLine(rpi,outBuff);         // RPi term program understands.
                                        // Sent with CR
      }
      if(!strcmp(inBuff,"gethumid"))
      {
        sprintf(outBuff,"%d",Hum/10);
        writeLine(rpi,outBuff);
      }                
      print("%s\n",inBuff);     // Local Debug         
  }  
}

/* XBee comms COG */
void talkXBee()
{
  /*                     Rx Tx         */
  xbee = fdserial_open(17,18,0,9600);
  pause(250);
/* These two lines are not showing up at start up ??? */
  writeStr(xbee,"Welcome\n");
  writeLine(xbee,"Type 'help' for menu.");
  ///////////////  
  sRTC_Start();
  
  sRTC_Update();
  sRTC_Set24hourmode(1);
  sRTC_Update();


  //pause(1000);
  writeStr(xbee,"#");   // This is not shwoing up at startup ??? ////
  char inBuff[40];
  
  while(1)
  {
    writeStr(xbee,"#");
    readStr(xbee, inBuff, 40);
    if(!strcmp(inBuff,"tempf"))
    {
      writeStr(xbee,"Temperature: ");
      writeDec(xbee,((TmpF/10)-3));
      writeLine(xbee," F");
    }
    else if(!strcmp(inBuff,"humid"))
    {
      writeStr(xbee,"Humidity: ");
      writeDec(xbee,Hum/10);
      writeLine(xbee,"%");
    }
    else if(!strcmp(inBuff,"setclock"))
    {
      writeLine(xbee,"Month,Day,Year XX,dow,Hour,Minutes,Seconds\n");
      writeLine(xbee,"dow: 0-Sun,1-Mon,2-Tue,3-Wed,4-Thu,5-Fri,6-Sat\n");
      Set_Clock();
    }
    else if(!strcmp(inBuff,"time"))
    {
      GetTime();
    }
    else if (!strcmp(inBuff, "date"))
    {
      GetTD();
    }
    else if (!strcmp(inBuff, "settime"))
    {
      writeLine(xbee,"Hour,Minute,Second");
      Set_Time();
    }
    else if(!strcmp(inBuff, "help"))
    {
      xbeeHelp();
    }                  
    else
    {
      writeLine(xbee,"Unknown Command");
    }            
  }      
  
}


int prompttime(char *prompt)
{
    int rc = 0;
    char *endp;
    char buffer[BUFFERLEN];
    
    do {
        dprinti(xbee,"Enter %s:  ",prompt);
        readStr(xbee, buffer, 10);
        rc = strtol(buffer, &endp, 10);
        if(endp == buffer)
        {
          if('\n' == *endp)
          {
            rc = 0;
            break;
          }
          dprinti(xbee,"Invalid entry \"%c.....\" Please enter a number.\n",*endp);           
        }          
      
    } while (endp == buffer);
    return rc;
}

void Set_Clock()
{
    int month,day,year,dow,hour,minute,second;
    month = prompttime("Month");
    day = prompttime("Day");
    year = prompttime("Year");
    dow = prompttime("DayOfWeek");
    hour = prompttime("Hour");
    minute = prompttime("Minute");
    second = prompttime("Second");
    
    dprinti(xbee,"\n%d %d %d %d %d %d %d\n",month,day,year,dow,hour,minute,second);
    sRTC_Setdatetime(month,day,year,dow,hour,minute,second);
    sRTC_Update();        // This is neccessary
}

void GetTime()
{
    int result, result1, result2;
    sRTC_Update();  // This is neccessary
    result = sRTC_Gethour();
    result1 = sRTC_Getminutes();
    result2 = sRTC_Getseconds();
    dprinti(xbee,"%d:%d:%d\n",result,result1,result2);  
}

void GetTD()
{
    int result, result1, result2;
    sRTC_Update();   // This is neccessary
    result = sRTC_Getmonth();
    result1 = sRTC_Getday();
    result2 = sRTC_Getyear();
    dprinti(xbee,"%d/%d/%d\n",result,result1,result2);  
}

void Set_Time()
{
    int hour,minute,second;
    hour = prompttime("Hour");
    minute = prompttime("Minute");
    second = prompttime("Second");
    dprinti(xbee,"\n%d %d %d\n",hour,minute,second);
    sRTC_Settime(hour,minute,second);
    sRTC_Update();  
}

void xbeeHelp()
{
  writeLine(xbee,"Menu - help, tempf, humid, setclock, settime");
  writeLine(xbee,"       time, date ");
}        
   
  
/**************************************/
/* Sensirion sht11 COG */
/* Drives the Sensirion sht11 every second for data refresh of module. */
void Sensirion()
{
// Moved to global declare area.
//long Hum;
//long TmpC;
//long TmpF;
//long Status;

  // Add startup code here.
  // Assign the address of our structure to the pointer
  pMyVals = &MyVals;
  high(16);   // Local Debug
  pause(1000);
  Start(DatPin,ClkPin,pMyVals);
  Status = ReadStatusReg();  
  while(1)
  {
    Measure();              // Get data from sht11 module.
    Hum = pMyVals->rhlin;   // % Humidity
    TmpC = pMyVals->tempC;  // C degrees
    TmpF = pMyVals->tempF;  // F degrees
    pause(1000);            // Pause for one second.

  }  
}


/*    Original Debug code, for comm with the freeBasic
      program on the Raspberry Pi.
      // Wait for key press from simpleide terminal
      getStr(locBuff, 40);
      writeLine(rpi,locBuff);  // Send it with CR
      // Displays incoming, up to 40 chars.
      readStr(rpi, inBuff, 40);
      print("%s\n",inBuff); 
*/